print '---------------------------------------- Trend Micro Canada ---------------------------------------';
print '******************* Executing Truncation Script ********************';
print ' ';
print ' ';
print 'SQL Server Remote Query Timeout=$(REMOTE_QUERY_TIMEOUT)';
print 'This script will operate on database=$(TENANT_DB_NAME)';
GO
print ' ';
print ' ';
print '---------------------------------------- Setting  remote query timeout to 0 ---------------------------------------';
EXEC sp_configure 'remote query timeout', '0';
GO
RECONFIGURE;
GO

print ' ';
print '---------------------------------------- Prevented sessions from re-establishing connection ----------------------------------------';
print ' ';

USE [master];
GO

declare @execSql varchar(2000);
declare @databaseName varchar(100);
SET @databaseName = '$(TENANT_DB_NAME)';
SET @execSql = '';
SELECT  @execSql = @execSql + 'KILL ' + cast(spid as varchar(100)) + '; ' FROM    master.dbo.sysprocesses WHERE (db_name(dbid) = @databaseName AND DBID <> 0 AND spid <> @@spid);

SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME)] SET SINGLE_USER WITH ROLLBACK IMMEDIATE; '; 
SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME)] MODIFY NAME = [$(TENANT_DB_NAME_NEW)]; '; 
SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME_NEW)] SET MULTI_USER WITH ROLLBACK IMMEDIATE; ';
print @execSql;

print '  ';
print '  ';
print '---------------------------------------- New DB Name [$(TENANT_DB_NAME_NEW)] ----------------------------------------';
print '  ';
print '  ';


EXEC(@execSql);
GO

print ' ';
print '---------------------------------------- Prevented sessions from re-establishing connections ----------------------------------------';
print ' ';




print '  ';
print '  ';
print '  ';
print '  ';
print '************************************************************ Executing Migration Simulation Script ************************************************************';
print '  ';
print '  ';
print '  ';
print '  ';
GO















-- ///////////////////////////////////////////////////////////////////////////
--
-- File Name    : DeepSecurityDatabaseBigintMigrationScriptTableTruncation.sql
-- File Version : 1.2.1006
--
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '---------------------------------------------------------------------------'
print '@@@ SQL Script Information'
print '---------------------------------------------------------------------------'
print ''
print '    ' + 'File Name    : DeepSecurityDatabaseBigintMigrationScriptTableTruncation.sql'
print '    ' + 'File Version : 1.2.1006'
go

-- ///////////////////////////////////////////////////////////////////////////
-- Modify this line to correct DSM database name.
-- ///////////////////////////////////////////////////////////////////////////
use [$(TENANT_DB_NAME_NEW)]
go

-- ///////////////////////////////////////////////////////////////////////////
-- Create table type for upgrade specification.
-- ///////////////////////////////////////////////////////////////////////////
create type tbl_upgrade_spec as table
    (pk_table_name nvarchar(255)
    ,pk_column_name nvarchar(255)
    ,pk_datatype nvarchar(255)
    ,nullable nvarchar(3));
go

-- ///////////////////////////////////////////////////////////////////////////
-- Create stored procedure for updating table schema
-- ///////////////////////////////////////////////////////////////////////////
create procedure TruncateTableContent
    @upgrade_spec tbl_upgrade_spec readonly
as
-- ///////////////////////////////////////////////////////////////////////////
declare @begin datetime = getdate();
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '---------------------------------------------------------------------------'
print '@@@ Deep Security Database Truncation Script >>> Table Truncation Running  '
print '---------------------------------------------------------------------------'
print '@@@ Start time : ' + convert(nvarchar(max), @begin, 21);
print '---------------------------------------------------------------------------'
-- ///////////////////////////////////////////////////////////////////////////
declare @progressTotal      int = 16;
declare @progressCurrent    int = 0;
declare @progressSubTotal   int = 0;
declare @progressSubCurrent int = 0;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '01. Define table to store parsed';
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '02. Determined Database Information'
-- ///////////////////////////////////////////////////////////////////////////
declare @dbname nvarchar(255);
declare @dbid nvarchar(255);
declare @schemaname nvarchar(255);
declare @schemaid nvarchar(255);
declare @editionID nvarchar(255);
declare @recovery_model nvarchar(255);
declare cur_dbinfo cursor for select db_name(), db_id(), schema_name(), schema_id(), cast(serverproperty('EditionID') as nvarchar(255));
open cur_dbinfo;
fetch next from cur_dbinfo into @dbname, @dbid, @schemaname, @schemaid, @editionID;
close cur_dbinfo;
deallocate cur_dbinfo;
declare cur_recovery_model cursor for select recovery_model_desc from sys.databases where name = @dbname;
open cur_recovery_model;
fetch next from cur_recovery_model into @recovery_model;
close cur_recovery_model;
deallocate cur_recovery_model;
print ''
print '    Information - ' + '@dbname: ' + @dbname + ', @dbid: ' + @dbid + ', @schemaname: ' + @schemaname + ', @schemaid: ' + @schemaid + ', @recovery_model: ' + @recovery_model;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '03. Reconfigure recovery mode to SIMPLE'
-- ///////////////////////////////////////////////////////////////////////////
declare @alterdbrecovery_simple nvarchar(2000) = 'alter database "' + @dbname + '" set recovery SIMPLE';
print ''
print '    Command : ' + @alterdbrecovery_simple;
exec(@alterdbrecovery_simple);
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '04. Determine primary keys'
-- ///////////////////////////////////////////////////////////////////////////
declare @pk_info table (pk_name nvarchar(255)
                       ,table_name nvarchar(255)
                       ,column_name nvarchar(255)
                       ,datatype nvarchar(255));
insert into @pk_info (pk_name, table_name, column_name, datatype)
select distinct a.CONSTRAINT_NAME, a.TABLE_NAME, b.COLUMN_NAME, c.pk_datatype
from INFORMATION_SCHEMA.TABLE_CONSTRAINTS a, INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE b, @upgrade_spec c
where a.TABLE_CATALOG = b.TABLE_CATALOG and a.TABLE_SCHEMA = b.TABLE_SCHEMA
and a.TABLE_NAME = b.TABLE_NAME
and b.TABLE_NAME = c.pk_table_name
and a.CONSTRAINT_NAME = b.CONSTRAINT_NAME
and a.CONSTRAINT_TYPE = 'PRIMARY KEY';
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '05. Determine foreign keys and indexes'
-- ///////////////////////////////////////////////////////////////////////////
declare cur_pk cursor for select pk_name, table_name, column_name, datatype from @pk_info;
declare @temp_pk_pk_name nvarchar(255);
declare @temp_pk_table_name nvarchar(255);
declare @temp_pk_column_name nvarchar(255);
declare @temp_pk_datatype nvarchar(255);
declare @fk_info table (fk_name nvarchar(255)
                       ,fk_table_name nvarchar(255)
                       ,fk_column_name nvarchar(255)
                       ,pk_name nvarchar(255)
                       ,pk_table_name nvarchar(255)
                       ,pk_column_name nvarchar(255)
                       ,fk_delete_rule nvarchar(20)
                       ,datatype nvarchar(255));
-- Note that this presumes that there is only one column per primary key (and thus foreign keys as well)
insert into @fk_info (fk_name, fk_table_name, fk_column_name, pk_table_name, pk_column_name, pk_name, fk_delete_rule, datatype)
select distinct rc.CONSTRAINT_NAME, fk_tc.TABLE_NAME, fk_ccu.column_name, rc.UNIQUE_CONSTRAINT_NAME, pk.table_name, pk.column_name, rc.delete_rule, pk.datatype
from INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS rc
    ,@pk_info pk
    ,INFORMATION_SCHEMA.TABLE_CONSTRAINTS fk_tc
    ,INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE fk_ccu
where rc.UNIQUE_CONSTRAINT_NAME = pk.pk_name
and rc.CONSTRAINT_CATALOG = @dbname and rc.CONSTRAINT_SCHEMA = @schemaname
and rc.CONSTRAINT_CATALOG = fk_tc.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_tc.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_tc.CONSTRAINT_NAME
and rc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME
and fk_tc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and fk_tc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and fk_tc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME;
declare @nonkey_indexes table (index_name nvarchar(255)
                              ,index_table nvarchar(255)
                              ,index_column nvarchar(255)
                              ,is_unique int
                              ,is_unique_constraint int
                              ,is_descending nvarchar(10)
                              ,type_desc nvarchar(255));
insert into @nonkey_indexes
select ind.name, t.name, col.name, ind.is_unique, ind.is_unique_constraint, case when ic.is_descending_key = 0 then 'ASC' else 'DESC' end, ind.type_desc
from   sys.indexes ind, sys.index_columns ic, sys.columns col, sys.tables t
where ind.is_primary_key = 0 and ind.object_id = ic.object_id and ind.index_id = ic.index_id
and ic.object_id = col.object_id and ic.column_id = col.column_id and ind.object_id = t.object_id
and t.is_ms_shipped = 0
and t.name + '|' + col.name in (select pku.pk_table_name + '|' + pku.pk_column_name from @upgrade_spec pku)
order by t.name, ind.name, ind.index_id, ic.index_column_id;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '06. Determine all columns to be altered, which should just be what is specified in the original alter_column_spec but include any additionally directly referencing fks in case any were manually specified'
-- ///////////////////////////////////////////////////////////////////////////
declare cur_nonkey_indexes cursor for select index_name, index_table, index_column, is_unique, is_unique_constraint, is_descending, type_desc from @nonkey_indexes;
declare @temp_nonkeyindex_name nvarchar(255);
declare @temp_nonkeyindex_table nvarchar(255);
declare @temp_nonkeyindex_column nvarchar(255);
declare @temp_nonkeyindex_is_unique int;
declare @temp_nonkeyindex_is_unique_constraint int;
declare @temp_nonkeyindex_is_descending nvarchar(10);
declare @temp_nonkey_type_desc nvarchar(255);
declare @alter_column_table table (table_name nvarchar(255)
                                  ,column_name nvarchar(255)
                                  ,datatype nvarchar(255)
                                  ,nullable nvarchar(3));
insert into @alter_column_table (table_name, column_name, datatype, nullable)
select pk_table_name, pk_column_name, pk_datatype, nullable from @upgrade_spec
union
select fk_tc.TABLE_NAME, fk_ccu.COLUMN_NAME, upg.pk_datatype, upg.nullable
from INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS rc,
     INFORMATION_SCHEMA.TABLE_CONSTRAINTS fk_tc,
     INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE fk_ccu,
     INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE rcu,
     @upgrade_spec upg
where rc.CONSTRAINT_CATALOG = @dbname and rc.CONSTRAINT_SCHEMA = @schemaname
and rc.CONSTRAINT_CATALOG = fk_tc.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_tc.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_tc.CONSTRAINT_NAME
and rc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME
and fk_tc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and fk_tc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and fk_tc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME
and rc.UNIQUE_CONSTRAINT_CATALOG = rcu.CONSTRAINT_CATALOG and rc.UNIQUE_CONSTRAINT_SCHEMA = rcu.CONSTRAINT_SCHEMA and rc.UNIQUE_CONSTRAINT_NAME = rcu.CONSTRAINT_NAME
and rcu.TABLE_NAME + '|' + rcu.COLUMN_NAME = upg.pk_table_name + '|' + upg.pk_column_name;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '07. Drop foreign keys'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @fk_info;
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
declare cur_alter_column_distinct_tables cursor for select distinct table_name from @alter_column_table;
declare @temp_alter_column_distinct_table_name nvarchar(255);
declare @temp_fk_name nvarchar(255);
declare @temp_fk_table nvarchar(255);
declare @temp_fk_column nvarchar(255);
declare @temp_fk_pk_table nvarchar(255);
declare @temp_fk_pk_column nvarchar(255);
declare @temp_fk_pk_name nvarchar(255);
declare @temp_fk_delete_rule nvarchar(255);
declare @temp_fk_datatype nvarchar(255);
declare cur_fk_tableInfo cursor for select distinct fk_name, fk_table_name, fk_column_name, pk_table_name, pk_column_name, pk_name, fk_delete_rule, datatype from @fk_info;
open cur_fk_tableInfo;
fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_pk_name, @temp_fk_delete_rule, @temp_fk_datatype;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @drop_constraint_cmd nvarchar(255) = 'alter table ' + @temp_fk_table + ' drop constraint ' + @temp_fk_name + ';';
    print '    running drop constraint command: ' + @drop_constraint_cmd;
    exec(@drop_constraint_cmd);
    fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_pk_name, @temp_fk_delete_rule, @temp_fk_datatype;
end
close cur_fk_tableInfo;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '08. Drop other indexes'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @nonkey_indexes
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
declare @dropped_uniqueKeyConstraints table (constraint_name nvarchar(2000)
                                            ,table_name nvarchar(2000)
                                            ,column_name nvarchar(2000));
open cur_nonkey_indexes;
fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
while @@FETCH_STATUS = 0
begin
    if (@temp_nonkeyindex_is_unique_constraint = 1)
    begin
        -- ///////////////////////////////////////////////////////////////////////////
        set @progressSubCurrent = @progressSubCurrent + 1;
        print ''
        print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
        print ''
        -- ///////////////////////////////////////////////////////////////////////////
        declare @drop_unique_constraint_cmd nvarchar(255) = 'alter table ' + @temp_nonkeyindex_table + ' drop constraint ' + @temp_nonkeyindex_name;
        print '    running drop unique constraint command: ' + @drop_unique_constraint_cmd;
        exec(@drop_unique_constraint_cmd);
        insert into @dropped_uniqueKeyConstraints(constraint_name, table_name, column_name)
        values( @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column);
    end;
    fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
end;
close cur_nonkey_indexes;
declare @dropped_nonKeyIndexes table (index_name nvarchar(2000)
                                     ,type_desc nvarchar(2000)
                                     ,table_name nvarchar(2000)
                                     ,column_name nvarchar(2000)
                                     ,is_descending nvarchar(10));
open cur_nonkey_indexes;
fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
while @@FETCH_STATUS = 0
begin
    if (@temp_nonkeyindex_is_unique_constraint = 0)
    begin
        -- ///////////////////////////////////////////////////////////////////////////
        set @progressSubCurrent = @progressSubCurrent + 1;
        print ''
        print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
        print ''
        -- ///////////////////////////////////////////////////////////////////////////
        print '    dropping nonkey index ' + @temp_nonkeyindex_name + ': ' + @temp_nonkeyindex_table + '.' + @temp_nonkeyindex_column;
        declare @drop_nonkey_index_cmd nvarchar(255) = 'drop index ' + @temp_nonkeyindex_name + ' on ' + @temp_nonkeyindex_table + ';';
        print '    running drop index command: ' + @drop_nonkey_index_cmd;
        exec(@drop_nonkey_index_cmd);
        insert into @dropped_nonKeyIndexes(index_name, type_desc, table_name, column_name, is_descending)
        values( @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending);
    end
    fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
end
close cur_nonkey_indexes;
deallocate cur_nonkey_indexes;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '09. Drop primary keys'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @pk_info
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
open cur_pk;
fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @drop_pk_cmd nvarchar(255) = 'alter table ' + @temp_pk_table_name + ' drop constraint ' + @temp_pk_pk_name + ';';
    print '    running drop pk command: ' + @drop_pk_cmd;
    exec(@drop_pk_cmd);
    fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
end
close cur_pk;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '10. Truncating all related tables'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(distinct table_name) from @alter_column_table
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
if (@editionID <> '-1592396055' and @editionID <> '-133711905') -- don't do this for 'express editions, as they impose an unchangeable 10 GB database size limit in which case this copying, done for performance, isn't so necessary.
begin
    open cur_alter_column_distinct_tables;
    fetch next from cur_alter_column_distinct_tables into @temp_alter_column_distinct_table_name;
    while @@FETCH_STATUS = 0
    begin
        -- ///////////////////////////////////////////////////////////////////////////
        set @progressSubCurrent = @progressSubCurrent + 1;
        print ''
        print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
        print ''
        -- ///////////////////////////////////////////////////////////////////////////
        if (@temp_alter_column_distinct_table_name = 'systemevents')
        begin
            declare @truncate_table_cmd_for_systemevent nvarchar(max) = 'truncate table activehosterrors; '
                                                                      + 'truncate table agentevents; '
                                                                      + 'truncate table autodiagnostic; '
                                                                      + 'truncate table systemeventhistory; '
                                                                      + 'truncate table ' + @temp_alter_column_distinct_table_name + ';';
            print '    truncate table for systemevents with all related tables: ' + @truncate_table_cmd_for_systemevent;
            exec(@truncate_table_cmd_for_systemevent);
        end
        else
        begin
            declare @truncate_table_cmd nvarchar(255) = 'truncate table ' + @temp_alter_column_distinct_table_name + ';';
            print '    truncate table: ' + @truncate_table_cmd;
            exec(@truncate_table_cmd);
        end
        fetch next from cur_alter_column_distinct_tables into @temp_alter_column_distinct_table_name;
    end
    close cur_alter_column_distinct_tables;
    deallocate cur_alter_column_distinct_tables;
end
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '11. Recreate primary keys'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @pk_info
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
open cur_pk;
fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @create_pk_cmd nvarchar(255) = 'alter table ' + @temp_pk_table_name + ' add constraint ' + @temp_pk_pk_name + ' primary key(' + @temp_pk_column_name + ');';
    print '    running create pk command: ' + @create_pk_cmd;
    exec(@create_pk_cmd);
    fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
end
close cur_pk;
deallocate cur_pk;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '12. Recreate foreign keys'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @fk_info
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
open cur_fk_tableInfo;
fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_name, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_delete_rule, @temp_fk_datatype;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @create_fk_cmd nvarchar(255) = 'alter table ' + @temp_fk_table + ' with check add constraint ' + @temp_fk_name + ' foreign key(' + @temp_fk_column + ') references ' + @temp_fk_pk_table + '(' + @temp_fk_pk_column + ')';
    if (@temp_fk_delete_rule='CASCADE')
    begin
        set @create_fk_cmd = @create_fk_cmd + ' on delete cascade';
    end;
    set @create_fk_cmd = @create_fk_cmd + ';';
    print '    running create fk command: ' + @create_fk_cmd;
    exec(@create_fk_cmd);
    fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_name, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_delete_rule, @temp_fk_datatype;
end
close cur_fk_tableInfo;
deallocate cur_fk_tableInfo;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '13. Recreate uniquekeys for other non-primary key indexes'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @dropped_uniqueKeyConstraints
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
declare cur_dropped_uniqueKeyConstraints cursor for
select constraint_name, table_name, column_name from @dropped_uniqueKeyConstraints;
open cur_dropped_uniqueKeyConstraints;
fetch next from cur_dropped_uniqueKeyConstraints into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @create_unique_constraint_cmd nvarchar(255) = 'alter table ' + @temp_nonkeyindex_table + ' add constraint ' + @temp_nonkeyindex_name + ' unique(' + @temp_nonkeyindex_column + ');';
    print '    running create unique constraint command: ' + @create_unique_constraint_cmd;
    exec(@create_unique_constraint_cmd);
    fetch next from cur_dropped_uniqueKeyConstraints into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column;
end
close cur_dropped_uniqueKeyConstraints;
deallocate cur_dropped_uniqueKeyConstraints;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '14. Recreate any leftover nonkey indexes'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @dropped_nonKeyIndexes
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
declare cur_dropped_nonkeyIndexes cursor for
select index_name, type_desc, table_name, column_name, is_descending from @dropped_nonKeyIndexes;
open cur_dropped_nonkeyIndexes;
fetch next from cur_dropped_nonkeyIndexes into @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @create_nonkeyindex_cmd nvarchar(255) = 'create ' + @temp_nonkey_type_desc + ' index ' + @temp_nonkeyindex_name + ' on ' + @temp_nonkeyindex_table + '(' + @temp_nonkeyindex_column + + ' ' + @temp_nonkeyindex_is_descending + ');';
    print '    running create index command: ' + @create_nonkeyindex_cmd;
    exec(@create_nonkeyindex_cmd);
    fetch next from cur_dropped_nonkeyIndexes into @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending;
end
close cur_dropped_nonkeyIndexes;
deallocate cur_dropped_nonkeyIndexes;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '15. Reset database recovery mode'
-- ///////////////////////////////////////////////////////////////////////////
declare @resetdbrecovery_bulklogged nvarchar(2000) = 'alter database "' + @dbname + '" set recovery ' + @recovery_model;
print ''
print '    Command : ' + @resetdbrecovery_bulklogged;
exec(@resetdbrecovery_bulklogged);
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '16. Table truncation completed successfully'
-- ///////////////////////////////////////////////////////////////////////////
declare @end datetime = getdate();
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '---------------------------------------------------------------------------'
print '@@@ Deep Security Database Truncation Script >>> Table Truncation Completed'
print '---------------------------------------------------------------------------'
print '@@@ Completed time : ' + convert(nvarchar(max), @end, 21);
print '---------------------------------------------------------------------------'
print '@@@ Total Table Truncation Time in seconds : ' + cast(datediff(SECOND, @begin, @end) as nvarchar);
print '---------------------------------------------------------------------------'
go

-- ///////////////////////////////////////////////////////////////////////////
-- The main procedure start
-- ///////////////////////////////////////////////////////////////////////////
set nocount on
go

-- ///////////////////////////////////////////////////////////////////////////
-- Define the column specification
declare @altercolumnspec nvarchar(max) = 'systemevents.TargetID:bigint:null|antimalwareevents.AntiMalwareEventID:bigint|antimalwarequarantinedfile.AntiMalwareEventID:bigint|antimalwarespywareitems.AntiMalwareEventID:bigint:null|webreputationevents.WebReputationEventID:bigint|antimalwareeventhistory.AntiMalwareEventHistoryID:bigint|antimalwareeventhistory.AntiMalwareEventID:bigint:null|webreputationeventhistory.WebReputationEventHistoryID:bigint|webreputationeventhistory.WebReputationEventID:bigint:null|integrityevents.IntegrityEventID:bigint|integrityeventhistory.IntegrityEventHistoryID:bigint|integrityeventhistory.IntegrityEventID:bigint:null|loginspectionevents.LogInspectionEventID:bigint|loginspectioneventhistory.LogInspectionEventHistoryID:bigint|loginspectioneventhistory.LogInspectionEventID:bigint:null|packetlogs.PacketLogID:bigint|packetlogdatas.PacketLogDataID:bigint|payloadlogs.PayloadLogID:bigint|payloadlogdatas.PayloadLogDataID:bigint|packetloghistory.PacketLogHistoryID:bigint|packetloghistory.PacketLogID:bigint:null|payloadloghistory.PayloadLogHistoryID:bigint|payloadloghistory.PayloadLogID:bigint:null|entitys.EntityID:bigint|attribute2s.EntityID:bigint|supersededentitys.WasEntityID:bigint';
--declare @altercolumnspec nvarchar(max) = 'systemevents.TargetID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'antimalwareevents.AntiMalwareEventID:bigint|antimalwarequarantinedfile.AntiMalwareEventID:bigint|antimalwarespywareitems.AntiMalwareEventID:bigint:null|antimalwareeventhistory.AntiMalwareEventHistoryID:bigint|antimalwareeventhistory.AntiMalwareEventID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'webreputationevents.WebReputationEventID:bigint|webreputationeventhistory.WebReputationEventHistoryID:bigint|webreputationeventhistory.WebReputationEventID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'integrityevents.IntegrityEventID:bigint|integrityeventhistory.IntegrityEventHistoryID:bigint|integrityeventhistory.IntegrityEventID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'loginspectionevents.LogInspectionEventID:bigint|loginspectioneventhistory.LogInspectionEventHistoryID:bigint|loginspectioneventhistory.LogInspectionEventID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'packetlogs.PacketLogID:bigint|packetlogdatas.PacketLogDataID:bigint|packetloghistory.PacketLogHistoryID:bigint|packetloghistory.PacketLogID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'payloadlogs.PayloadLogID:bigint|payloadlogdatas.PayloadLogDataID:bigint|payloadloghistory.PayloadLogHistoryID:bigint|payloadloghistory.PayloadLogID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'entitys.EntityID:bigint|attribute2s.EntityID:bigint|supersededentitys.WasEntityID:bigint';
-- ///////////////////////////////////////////////////////////////////////////
declare @upgrade_spec as tbl_upgrade_spec;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '----------------------------------------------------------------------------'
print '@@@ Deep Security Database Truncation Script >>> Build Upgrade Specification'
print '----------------------------------------------------------------------------'
-- ///////////////////////////////////////////////////////////////////////////
declare @altercolumnentry nvarchar(512);
declare @altercolumnspecidx int = len(@altercolumnspec);
while (@altercolumnspecidx > 0)
begin
    set @altercolumnspecidx = charindex('|', @altercolumnspec);
    if (@altercolumnspecidx = 0)
    begin
        set @altercolumnentry = @altercolumnspec;
        set @altercolumnspec = '';
    end
    else
    begin
        set @altercolumnentry = substring(@altercolumnspec,1,@altercolumnspecidx-1);
        set @altercolumnspec = substring(@altercolumnspec,@altercolumnspecidx+1,len(@altercolumnspec)-@altercolumnspecidx);
    end
    print ''
    print '    current entry: ' + @altercolumnentry;
    print '    remainder: ' + @altercolumnspec;
    declare @tableEndPos int = charindex('.', @altercolumnentry);
    declare @columnEndPos int = charindex(':', @altercolumnentry);
    print '    tableEndPos: ' + cast(@tableEndPos as nvarchar(255));
    print '    columnEndPos: ' + cast(@columnEndPos as nvarchar(255));
    declare @altercolumnentry_tableName nvarchar(127) = substring(@altercolumnentry,1,@tableEndPos-1);
    declare @altercolumnentry_columnName nvarchar(127) = substring(@altercolumnentry,@tableEndPos+1,@columnEndPos-@tableEndPos-1);
    declare @altercolumnentry_datatype nvarchar(127) = substring(@altercolumnentry,@columnEndPos+1,len(@altercolumnentry)-@columnEndPos);
    declare @altercolumnentry_nullable nvarchar(3) = 'NO';
    if (substring(@altercolumnentry_datatype,len(@altercolumnentry_datatype)-4, 5) = ':null')
    begin
        set @altercolumnentry_nullable = 'YES';
        set @altercolumnentry_datatype = substring(@altercolumnentry_datatype,1, len(@altercolumnentry_datatype)-5);
    end
    -- ///////////////////////////////////////////////////////////////////////////
    if exists (select TABLE_NAME from INFORMATION_SCHEMA.COLUMNS where DATA_TYPE <> @altercolumnentry_datatype and TABLE_NAME = @altercolumnentry_tableName and COLUMN_NAME = @altercolumnentry_columnName)
    begin
        insert into @upgrade_spec(pk_table_name, pk_column_name, pk_datatype, nullable)
        values(@altercolumnentry_tableName,@altercolumnentry_columnName,@altercolumnentry_datatype, @altercolumnentry_nullable);
    end
    else
    begin
        print '    ' + 'The datatype of column ' + @altercolumnentry_columnName + ' in table ' + @altercolumnentry_tableName + ' has already been ' + @altercolumnentry_datatype
        print '    ' + 'Skip this item'
    end
    -- ///////////////////////////////////////////////////////////////////////////
end
-- ///////////////////////////////////////////////////////////////////////////
if exists (select pk_table_name from @upgrade_spec)
begin
    exec TruncateTableContent @upgrade_spec
end
else
begin
    print ''
    print '    ' + 'There are no items in upgrade specification table. No need to do any update on current database'
end
go

-- ///////////////////////////////////////////////////////////////////////////
-- Drop store procedure before we leave
drop procedure TruncateTableContent;
drop type tbl_upgrade_spec;
go

-- ///////////////////////////////////////////////////////////////////////////
set nocount off
go















print '  ';
print '  ';
print '  ';
print '  ';
print '******************** Finished Executing Truncation Script ********************';
print '  ';
print '  ';
print '  ';
print '  ';
GO






USE [master];
GO

declare @execSql varchar(2000);
declare @databaseName varchar(100);
SET @databaseName = '[$(TENANT_DB_NAME_NEW)]';
SET @execSql = '';
SELECT  @execSql = @execSql + 'KILL ' + cast(spid as varchar(100)) + '; ' FROM    master.dbo.sysprocesses WHERE (db_name(dbid) = @databaseName AND DBID <> 0 AND spid <> @@spid);
SET @execSql = @execSql + '; ';

SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME_NEW)] SET SINGLE_USER WITH ROLLBACK IMMEDIATE; '; 
SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME_NEW)] MODIFY NAME = [$(TENANT_DB_NAME)]; '; 
SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME)] SET MULTI_USER WITH ROLLBACK IMMEDIATE; ';

print '  ';
print '  ';
print '---------------------------------------- Reverted DB Name [$(TENANT_DB_NAME)] ----------------------------------------';
print '  ';
print '  ';

EXEC(@execSql);
GO


EXEC sp_configure 'remote query timeout', '$(REMOTE_QUERY_TIMEOUT)';
GO
RECONFIGURE;
GO

print '---------------------------------------- Sending Quit to SQLCMD ----------------------------------------';
GO

:quit
