print '---------------------------------------- Trend Micro Canada ---------------------------------------';
print '******************* Executing Migration Script ********************';
print ' ';
print ' ';
print 'SQL Server Remote Query Timeout=$(REMOTE_QUERY_TIMEOUT)';
print 'This script will operate on database=$(TENANT_DB_NAME)';
GO
print ' ';
print ' ';
print '---------------------------------------- Setting  remote query timeout to 0 ---------------------------------------';
EXEC sp_configure 'remote query timeout', '0';
GO
RECONFIGURE;
GO

print ' ';
print '---------------------------------------- Prevented sessions from re-establishing connection ----------------------------------------';
print ' ';

USE [master];
GO

declare @execSql varchar(2000);
declare @databaseName varchar(100);
SET @databaseName = '$(TENANT_DB_NAME)';
SET @execSql = '';
SELECT  @execSql = @execSql + 'KILL ' + cast(spid as varchar(100)) + '; ' FROM    master.dbo.sysprocesses WHERE (db_name(dbid) = @databaseName AND DBID <> 0 AND spid <> @@spid);

SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME)] SET SINGLE_USER WITH ROLLBACK IMMEDIATE; '; 
SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME)] MODIFY NAME = [$(TENANT_DB_NAME_NEW)]; '; 
SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME_NEW)] SET MULTI_USER WITH ROLLBACK IMMEDIATE; ';
print @execSql;

print '  ';
print '  ';
print '---------------------------------------- New DB Name [$(TENANT_DB_NAME_NEW)] ----------------------------------------';
print '  ';
print '  ';


EXEC(@execSql);
GO

print ' ';
print '---------------------------------------- Prevented sessions from re-establishing connections ----------------------------------------';
print ' ';
go




















-- ///////////////////////////////////////////////////////////////////////////
--
-- File Name    : DeepSecurityDatabaseBigintMigrationScript.sql
-- File Version : 1.3.1002
--
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '---------------------------------------------------------------------------'
print '@@@ SQL Script Information'
print '---------------------------------------------------------------------------'
print ''
print '    ' + 'File Name    : DeepSecurityDatabaseBigintMigrationScript.sql'
print '    ' + 'File Version : 1.3.1002'
go

-- ///////////////////////////////////////////////////////////////////////////
-- Modify this line to correct DSM database name.
-- ///////////////////////////////////////////////////////////////////////////
use [$(TENANT_DB_NAME_NEW)]
go

-- ///////////////////////////////////////////////////////////////////////////
-- Create table type for upgrade specification.
-- ///////////////////////////////////////////////////////////////////////////
create type tbl_upgrade_spec as table
    (pk_table_name nvarchar(255)
    ,pk_column_name nvarchar(255)
    ,pk_datatype nvarchar(255)
    ,nullable nvarchar(3));
go

-- ///////////////////////////////////////////////////////////////////////////
-- Create stored procedure for generating recovery command
-- ///////////////////////////////////////////////////////////////////////////
create procedure GenerateRecoveryCommand
    @upgrade_spec tbl_upgrade_spec readonly
as
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '---------------------------------------------------------------------------'
print '@@@ Deep Security Database Migration Script >>> Recovery Commands Preparing'
print '---------------------------------------------------------------------------'
-- ///////////////////////////////////////////////////////////////////////////
declare @dbname nvarchar(255);
declare @dbid nvarchar(255);
declare @schemaname nvarchar(255);
declare @schemaid nvarchar(255);
declare @editionID nvarchar(255);
declare @recovery_model nvarchar(255);
declare cur_dbinfo cursor for select db_name(), db_id(), schema_name(), schema_id(), cast(serverproperty('EditionID') as nvarchar(255));
open cur_dbinfo;
fetch next from cur_dbinfo into @dbname, @dbid, @schemaname, @schemaid, @editionID;
close cur_dbinfo;
deallocate cur_dbinfo;
declare cur_recovery_model cursor for select recovery_model_desc from sys.databases where name = @dbname;
open cur_recovery_model;
fetch next from cur_recovery_model into @recovery_model;
close cur_recovery_model;
deallocate cur_recovery_model;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Checking primary keys'
-- ///////////////////////////////////////////////////////////////////////////
declare @pk_info table (pk_name nvarchar(255)
                       ,table_name nvarchar(255)
                       ,column_name nvarchar(255)
                       ,datatype nvarchar(255));
insert into @pk_info (pk_name, table_name, column_name, datatype)
select distinct a.CONSTRAINT_NAME, a.TABLE_NAME, b.COLUMN_NAME, c.pk_datatype
from INFORMATION_SCHEMA.TABLE_CONSTRAINTS a, INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE b, @upgrade_spec c
where a.TABLE_CATALOG = b.TABLE_CATALOG and a.TABLE_SCHEMA = b.TABLE_SCHEMA
and a.TABLE_NAME = b.TABLE_NAME
and b.TABLE_NAME = c.pk_table_name
and a.CONSTRAINT_NAME = b.CONSTRAINT_NAME
and a.CONSTRAINT_TYPE = 'PRIMARY KEY';
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Checking foreign keys and indexes'
-- ///////////////////////////////////////////////////////////////////////////
declare cur_pk cursor for select pk_name, table_name, column_name, datatype from @pk_info;
declare @temp_pk_pk_name nvarchar(255);
declare @temp_pk_table_name nvarchar(255);
declare @temp_pk_column_name nvarchar(255);
declare @temp_pk_datatype nvarchar(255);
declare @fk_info table (fk_name nvarchar(255)
                       ,fk_table_name nvarchar(255)
                       ,fk_column_name nvarchar(255)
                       ,pk_name nvarchar(255)
                       ,pk_table_name nvarchar(255)
                       ,pk_column_name nvarchar(255)
                       ,fk_delete_rule nvarchar(20)
                       ,datatype nvarchar(255));
-- Note that this presumes that there is only one column per primary key (and thus foreign keys as well)
insert into @fk_info (fk_name, fk_table_name, fk_column_name, pk_table_name, pk_column_name, pk_name, fk_delete_rule, datatype)
select distinct rc.CONSTRAINT_NAME, fk_tc.TABLE_NAME, fk_ccu.COLUMN_NAME, rc.UNIQUE_CONSTRAINT_NAME, pk.table_name, pk.column_name, rc.delete_rule, pk.datatype
from INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS rc
    ,@pk_info pk
    ,INFORMATION_SCHEMA.TABLE_CONSTRAINTS fk_tc
    ,INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE fk_ccu
where rc.UNIQUE_CONSTRAINT_NAME = pk.pk_name
and rc.CONSTRAINT_CATALOG = @dbname and rc.CONSTRAINT_SCHEMA = @schemaname
and rc.CONSTRAINT_CATALOG = fk_tc.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_tc.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_tc.CONSTRAINT_NAME
and rc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME
and fk_tc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and fk_tc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and fk_tc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME;
declare @nonkey_indexes table (index_name nvarchar(255)
                              ,index_table nvarchar(255)
                              ,index_column nvarchar(255)
                              ,is_unique int
                              ,is_unique_constraint int
                              ,is_descending nvarchar(10)
                              ,type_desc nvarchar(255));
insert into @nonkey_indexes
select ind.name, t.name, col.name, ind.is_unique, ind.is_unique_constraint, case when ic.is_descending_key = 0 then 'ASC' else 'DESC' end, ind.type_desc
from   sys.indexes ind, sys.index_columns ic, sys.columns col, sys.tables t
where ind.is_primary_key = 0 and ind.object_id = ic.object_id and ind.index_id = ic.index_id
and ic.object_id = col.object_id and ic.column_id = col.column_id and ind.object_id = t.object_id
and t.is_ms_shipped = 0
and t.name + '|' + col.name in (select pku.pk_table_name + '|' + pku.pk_column_name from @upgrade_spec pku)
order by t.name, ind.name, ind.index_id, ic.index_column_id;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Checking all columns to be altered'
-- ///////////////////////////////////////////////////////////////////////////
declare cur_nonkey_indexes cursor for select index_name, index_table, index_column, is_unique, is_unique_constraint, is_descending, type_desc from @nonkey_indexes;
declare @temp_nonkeyindex_name nvarchar(255);
declare @temp_nonkeyindex_table nvarchar(255);
declare @temp_nonkeyindex_column nvarchar(255);
declare @temp_nonkeyindex_is_unique int;
declare @temp_nonkeyindex_is_unique_constraint int;
declare @temp_nonkeyindex_is_descending nvarchar(10);
declare @temp_nonkey_type_desc nvarchar(255);
declare @alter_column_table table (table_name nvarchar(255)
                                  ,column_name nvarchar(255)
                                  ,datatype nvarchar(255)
                                  ,nullable nvarchar(3));
insert into @alter_column_table (table_name, column_name, datatype, nullable)
select pk_table_name, pk_column_name, pk_datatype, nullable from @upgrade_spec
union
select fk_tc.TABLE_NAME, fk_ccu.COLUMN_NAME, upg.pk_datatype, upg.nullable
from INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS rc,
     INFORMATION_SCHEMA.TABLE_CONSTRAINTS fk_tc,
     INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE fk_ccu,
     INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE rcu,
     @upgrade_spec upg
where rc.CONSTRAINT_CATALOG = @dbname and rc.CONSTRAINT_SCHEMA = @schemaname
and rc.CONSTRAINT_CATALOG = fk_tc.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_tc.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_tc.CONSTRAINT_NAME
and rc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME
and fk_tc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and fk_tc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and fk_tc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME
and rc.UNIQUE_CONSTRAINT_CATALOG = rcu.CONSTRAINT_CATALOG and rc.UNIQUE_CONSTRAINT_SCHEMA = rcu.CONSTRAINT_SCHEMA and rc.UNIQUE_CONSTRAINT_NAME = rcu.CONSTRAINT_NAME
and rcu.TABLE_NAME + '|' + rcu.COLUMN_NAME = upg.pk_table_name + '|' + upg.pk_column_name;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Drop foreign keys'
print ''
-- ///////////////////////////////////////////////////////////////////////////
declare cur_alter_column_distinct_tables cursor for select distinct table_name from @alter_column_table;
declare @temp_alter_column_distinct_table_name nvarchar(255);
declare @temp_fk_name nvarchar(255);
declare @temp_fk_table nvarchar(255);
declare @temp_fk_column nvarchar(255);
declare @temp_fk_pk_table nvarchar(255);
declare @temp_fk_pk_column nvarchar(255);
declare @temp_fk_pk_name nvarchar(255);
declare @temp_fk_delete_rule nvarchar(255);
declare @temp_fk_datatype nvarchar(255);
declare cur_fk_tableInfo cursor for select distinct fk_name, fk_table_name, fk_column_name, pk_table_name, pk_column_name, pk_name, fk_delete_rule, datatype from @fk_info;
open cur_fk_tableInfo;
fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_pk_name, @temp_fk_delete_rule, @temp_fk_datatype;
while @@FETCH_STATUS = 0
begin
    declare @drop_constraint_cmd nvarchar(255) = 'alter table ' + @temp_fk_table + ' drop constraint ' + @temp_fk_name + ';';
    print '    ' + @drop_constraint_cmd;
    fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_pk_name, @temp_fk_delete_rule, @temp_fk_datatype;
end
close cur_fk_tableInfo;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Drop other indexes'
print ''
-- ///////////////////////////////////////////////////////////////////////////
declare @dropped_uniqueKeyConstraints table (constraint_name nvarchar(2000)
                                            ,table_name nvarchar(2000)
                                            ,column_name nvarchar(2000));
open cur_nonkey_indexes;
fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
while @@FETCH_STATUS = 0
begin
    if (@temp_nonkeyindex_is_unique_constraint = 1)
    begin
        declare @drop_unique_constraint_cmd nvarchar(255) = 'alter table ' + @temp_nonkeyindex_table + ' drop constraint ' + @temp_nonkeyindex_name;
        print '    ' + @drop_unique_constraint_cmd;
        insert into @dropped_uniqueKeyConstraints(constraint_name, table_name, column_name)
        values( @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column);
    end;
    fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
end;
close cur_nonkey_indexes;
declare @dropped_nonKeyIndexes table (index_name nvarchar(2000)
                                     ,type_desc nvarchar(2000)
                                     ,table_name nvarchar(2000)
                                     ,column_name nvarchar(2000)
                                     ,is_descending nvarchar(10));
open cur_nonkey_indexes;
fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
while @@FETCH_STATUS = 0
begin
    if (@temp_nonkeyindex_is_unique_constraint = 0)
    begin
        declare @drop_nonkey_index_cmd nvarchar(255) = 'drop index ' + @temp_nonkeyindex_name + ' on ' + @temp_nonkeyindex_table + ';';
        print '    ' + @drop_nonkey_index_cmd;
        insert into @dropped_nonKeyIndexes(index_name, type_desc, table_name, column_name, is_descending)
        values( @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending);
    end
    fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
end
close cur_nonkey_indexes;
deallocate cur_nonkey_indexes;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Drop primary keys'
print ''
-- ///////////////////////////////////////////////////////////////////////////
open cur_pk;
fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
while @@FETCH_STATUS = 0
begin
    declare @drop_pk_cmd nvarchar(255) = 'alter table ' + @temp_pk_table_name + ' drop constraint ' + @temp_pk_pk_name + ';';
    print '    ' + @drop_pk_cmd;
    fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
end
close cur_pk;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Recreate primary keys'
print ''
-- ///////////////////////////////////////////////////////////////////////////
open cur_pk;
fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
while @@FETCH_STATUS = 0
begin
    declare @create_pk_cmd nvarchar(255) = 'alter table ' + @temp_pk_table_name + ' add constraint ' + @temp_pk_pk_name + ' primary key(' + @temp_pk_column_name + ');';
    print '    ' + @create_pk_cmd;
    fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
end
close cur_pk;
deallocate cur_pk;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Recreate foreign keys'
print ''
-- ///////////////////////////////////////////////////////////////////////////
open cur_fk_tableInfo;
fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_name, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_delete_rule, @temp_fk_datatype;
while @@FETCH_STATUS = 0
begin
    declare @create_fk_cmd nvarchar(255) = 'alter table ' + @temp_fk_table + ' with check add constraint ' + @temp_fk_name + ' foreign key(' + @temp_fk_column + ') references ' + @temp_fk_pk_table + '(' + @temp_fk_pk_column + ')';
    if (@temp_fk_delete_rule='CASCADE')
    begin
        set @create_fk_cmd = @create_fk_cmd + ' on delete cascade';
    end;
    set @create_fk_cmd = @create_fk_cmd + ';';
    print '    ' + @create_fk_cmd;
    fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_name, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_delete_rule, @temp_fk_datatype;
end
close cur_fk_tableInfo;
deallocate cur_fk_tableInfo;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Recreate uniquekeys for other non-primary key indexes'
print ''
-- ///////////////////////////////////////////////////////////////////////////
declare cur_dropped_uniqueKeyConstraints cursor for
select constraint_name, table_name, column_name from @dropped_uniqueKeyConstraints;
open cur_dropped_uniqueKeyConstraints;
fetch next from cur_dropped_uniqueKeyConstraints into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column;
while @@FETCH_STATUS = 0
begin
    declare @create_unique_constraint_cmd nvarchar(255) = 'alter table ' + @temp_nonkeyindex_table + ' add constraint ' + @temp_nonkeyindex_name + ' unique(' + @temp_nonkeyindex_column + ');';
    print '    ' + @create_unique_constraint_cmd;
    fetch next from cur_dropped_uniqueKeyConstraints into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column;
end
close cur_dropped_uniqueKeyConstraints;
deallocate cur_dropped_uniqueKeyConstraints;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '*** Command Dumping >>> Recreate any leftover nonkey indexes'
print ''
-- ///////////////////////////////////////////////////////////////////////////
declare cur_dropped_nonkeyIndexes cursor for
select index_name, type_desc, table_name, column_name, is_descending from @dropped_nonKeyIndexes;
open cur_dropped_nonkeyIndexes;
fetch next from cur_dropped_nonkeyIndexes into @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending;
while @@FETCH_STATUS = 0
begin
    declare @create_nonkeyindex_cmd nvarchar(255) = 'create ' + @temp_nonkey_type_desc + ' index ' + @temp_nonkeyindex_name + ' on ' + @temp_nonkeyindex_table + '(' + @temp_nonkeyindex_column + + ' ' + @temp_nonkeyindex_is_descending + ');';
    print '    ' + @create_nonkeyindex_cmd;
    fetch next from cur_dropped_nonkeyIndexes into @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending;
end
close cur_dropped_nonkeyIndexes;
deallocate cur_dropped_nonkeyIndexes;
go

-- ///////////////////////////////////////////////////////////////////////////
-- Create stored procedure for updating table schema
-- ///////////////////////////////////////////////////////////////////////////
create procedure UpdateTableSchema
    @upgrade_spec tbl_upgrade_spec readonly
as
-- ///////////////////////////////////////////////////////////////////////////
declare @begin datetime = getdate();
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '---------------------------------------------------------------------------'
print '@@@ Deep Security Database Migration Script >>> Schema Migration Running   '
print '---------------------------------------------------------------------------'
print '@@@ Start time : ' + convert(nvarchar(max), @begin, 21);
print '---------------------------------------------------------------------------'
-- ///////////////////////////////////////////////////////////////////////////
declare @progressTotal      int = 19;
declare @progressCurrent    int = 0;
declare @progressSubTotal   int = 0;
declare @progressSubCurrent int = 0;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '01. Define table to store parsed';
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '02. Determined Database Information'
-- ///////////////////////////////////////////////////////////////////////////
declare @dbname nvarchar(255);
declare @dbid nvarchar(255);
declare @schemaname nvarchar(255);
declare @schemaid nvarchar(255);
declare @editionID nvarchar(255);
declare @recovery_model nvarchar(255);
declare cur_dbinfo cursor for select db_name(), db_id(), schema_name(), schema_id(), cast(serverproperty('EditionID') as nvarchar(255));
open cur_dbinfo;
fetch next from cur_dbinfo into @dbname, @dbid, @schemaname, @schemaid, @editionID;
close cur_dbinfo;
deallocate cur_dbinfo;
declare cur_recovery_model cursor for select recovery_model_desc from sys.databases where name = @dbname;
open cur_recovery_model;
fetch next from cur_recovery_model into @recovery_model;
close cur_recovery_model;
deallocate cur_recovery_model;
print ''
print '    Information - ' + '@dbname: ' + @dbname + ', @dbid: ' + @dbid + ', @schemaname: ' + @schemaname + ', @schemaid: ' + @schemaid + ', @recovery_model: ' + @recovery_model;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '03. Reconfigure recovery mode to simple'
-- ///////////////////////////////////////////////////////////////////////////
declare @alterdbrecovery_simple nvarchar(2000) = 'alter database "' + @dbname + '" set recovery simple';
print ''
print '    Command : ' + @alterdbrecovery_simple;
exec(@alterdbrecovery_simple);
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '04. Restoring lingering temporary tables existing in case of a previously failed upgrade attempt'
-- ///////////////////////////////////////////////////////////////////////////
-- this is not necessary if it's express edition, like steps 11 and 13 below, but letting things fall through just in case.
declare @restore_temp_table_name nvarchar(2000);
declare @restore_prev_temp_table_name nvarchar(2000)='';
declare @restore_src_prev_table_name nvarchar(2000);
declare @restore_temp_column_name nvarchar(4000);
declare @restore_temp_columns_str nvarchar(4000) = '';
declare cur_temptables cursor for select table_name, column_name from INFORMATION_SCHEMA.COLUMNS where TABLE_NAME like '%_dsmupgradetemp';
declare @restore_temp_row_number bigint;
declare @identity_on nvarchar(1024);
declare @identity_off nvarchar(1024);
--
open cur_temptables;
fetch next from cur_temptables into @restore_temp_table_name, @restore_temp_column_name;
while @@FETCH_STATUS = 0
begin
    declare @restore_src_table_name nvarchar(2000) = substring(@restore_temp_table_name, 1, len(@restore_temp_table_name)-15) ;
    if (@restore_prev_temp_table_name = '')
    begin
        set @restore_prev_temp_table_name = @restore_temp_table_name;
    end
    else if (@restore_temp_table_name <> @restore_prev_temp_table_name)
    begin
        set @restore_src_prev_table_name = substring(@restore_prev_temp_table_name, 1, len(@restore_prev_temp_table_name)-15) ;
        set @identity_on  = '';
        set @identity_off = '';
        if objectproperty(object_id(@restore_src_prev_table_name), 'TableHasIdentity') = 1
        begin
            set @identity_on  =  'set identity_insert ' + @restore_src_prev_table_name + ' on;  ';
            set @identity_off = ' set identity_insert ' + @restore_src_prev_table_name + ' off; '
                                + ' dbcc checkident  (' + @restore_src_prev_table_name + ',reseed); ';
        end
        print ''
        print @identity_on;
        print 'insert into ' + @restore_src_prev_table_name + ' (' + @restore_temp_columns_str + ') select ' + @restore_temp_columns_str + ' from ' + @restore_prev_temp_table_name;
        print @identity_off;
        exec( @identity_on
            + ' insert into ' + @restore_src_prev_table_name + ' (' + @restore_temp_columns_str + ') select ' + @restore_temp_columns_str + ' from ' + @restore_prev_temp_table_name
            + @identity_off );
        print 'drop table ' + @restore_prev_temp_table_name + '; ';
        exec( 'drop table ' + @restore_prev_temp_table_name + '; ');
        set @restore_temp_columns_str = '';
        set @restore_prev_temp_table_name = @restore_temp_table_name;
    end
    if (@restore_temp_columns_str <> '')
    begin
        set @restore_temp_columns_str = @restore_temp_columns_str + ',';
    end
    set @restore_temp_columns_str = @restore_temp_columns_str + '[' + @restore_temp_column_name + ']';
    fetch next from cur_temptables into @restore_temp_table_name, @restore_temp_column_name;
end
close cur_temptables;
deallocate cur_temptables;
if (@restore_prev_temp_table_name <> '')
begin
    set @restore_src_prev_table_name = substring(@restore_prev_temp_table_name, 1, len(@restore_prev_temp_table_name)-15) ;
    print ''
    set @identity_on  = '';
    set @identity_off = '';
    if objectproperty(object_id(@restore_src_prev_table_name), 'TableHasIdentity') = 1
    begin
        set @identity_on  =  'set identity_insert ' + @restore_src_prev_table_name + ' on;  ';
        set @identity_off = ' set identity_insert ' + @restore_src_prev_table_name + ' off; '
                            + ' dbcc checkident  (' + @restore_src_prev_table_name + ',reseed); ';
    end
    print @identity_on
    print 'insert into ' + @restore_src_prev_table_name + ' (' + @restore_temp_columns_str + ') select ' + @restore_temp_columns_str + ' from ' + @restore_prev_temp_table_name;
    print @identity_off
    exec( @identity_on
        + ' insert into ' + @restore_src_prev_table_name + ' (' + @restore_temp_columns_str + ') select ' + @restore_temp_columns_str + ' from ' + @restore_prev_temp_table_name
        + @identity_off );
    print 'drop table ' + @restore_prev_temp_table_name + '; ';
    exec( 'drop table ' + @restore_prev_temp_table_name + '; ');
end
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '05. Determine primary keys'
-- ///////////////////////////////////////////////////////////////////////////
declare @pk_info table (pk_name nvarchar(255)
                       ,table_name nvarchar(255)
                       ,column_name nvarchar(255)
                       ,datatype nvarchar(255));
insert into @pk_info (pk_name, table_name, column_name, datatype)
select distinct a.CONSTRAINT_NAME, a.TABLE_NAME, b.COLUMN_NAME, c.pk_datatype
from INFORMATION_SCHEMA.TABLE_CONSTRAINTS a, INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE b, @upgrade_spec c
where a.TABLE_CATALOG = b.TABLE_CATALOG and a.TABLE_SCHEMA = b.TABLE_SCHEMA
and a.TABLE_NAME = b.TABLE_NAME
and b.TABLE_NAME = c.pk_table_name
and a.CONSTRAINT_NAME = b.CONSTRAINT_NAME
and a.CONSTRAINT_TYPE = 'PRIMARY KEY';
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '06. Determine foreign keys and indexes'
-- ///////////////////////////////////////////////////////////////////////////
declare cur_pk cursor for select pk_name, table_name, column_name, datatype from @pk_info;
declare @temp_pk_pk_name nvarchar(255);
declare @temp_pk_table_name nvarchar(255);
declare @temp_pk_column_name nvarchar(255);
declare @temp_pk_datatype nvarchar(255);
declare @fk_info table (fk_name nvarchar(255)
                       ,fk_table_name nvarchar(255)
                       ,fk_column_name nvarchar(255)
                       ,pk_name nvarchar(255)
                       ,pk_table_name nvarchar(255)
                       ,pk_column_name nvarchar(255)
                       ,fk_delete_rule nvarchar(20)
                       ,datatype nvarchar(255));
-- Note that this presumes that there is only one column per primary key (and thus foreign keys as well)
insert into @fk_info (fk_name, fk_table_name, fk_column_name, pk_table_name, pk_column_name, pk_name, fk_delete_rule, datatype)
select distinct rc.CONSTRAINT_NAME, fk_tc.TABLE_NAME, fk_ccu.COLUMN_NAME, rc.UNIQUE_CONSTRAINT_NAME, pk.table_name, pk.column_name, rc.delete_rule, pk.datatype
from INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS rc
    ,@pk_info pk
    ,INFORMATION_SCHEMA.TABLE_CONSTRAINTS fk_tc
    ,INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE fk_ccu
where rc.UNIQUE_CONSTRAINT_NAME = pk.pk_name
and rc.CONSTRAINT_CATALOG = @dbname and rc.CONSTRAINT_SCHEMA = @schemaname
and rc.CONSTRAINT_CATALOG = fk_tc.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_tc.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_tc.CONSTRAINT_NAME
and rc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME
and fk_tc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and fk_tc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and fk_tc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME;
declare @nonkey_indexes table (index_name nvarchar(255)
                              ,index_table nvarchar(255)
                              ,index_column nvarchar(255)
                              ,is_unique int
                              ,is_unique_constraint int
                              ,is_descending nvarchar(10)
                              ,type_desc nvarchar(255));
insert into @nonkey_indexes
select ind.name, t.name, col.name, ind.is_unique, ind.is_unique_constraint, case when ic.is_descending_key = 0 then 'ASC' else 'DESC' end, ind.type_desc
from   sys.indexes ind, sys.index_columns ic, sys.columns col, sys.tables t
where ind.is_primary_key = 0 and ind.object_id = ic.object_id and ind.index_id = ic.index_id
and ic.object_id = col.object_id and ic.column_id = col.column_id and ind.object_id = t.object_id
and t.is_ms_shipped = 0
and t.name + '|' + col.name in (select pku.pk_table_name + '|' + pku.pk_column_name from @upgrade_spec pku)
order by t.name, ind.name, ind.index_id, ic.index_column_id;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '07. Determine all columns to be altered, which should just be what is specified in the original alter_column_spec but include any additionally directly referencing fks in case any were manually specified'
-- ///////////////////////////////////////////////////////////////////////////
declare cur_nonkey_indexes cursor for select index_name, index_table, index_column, is_unique, is_unique_constraint, is_descending, type_desc from @nonkey_indexes;
declare @temp_nonkeyindex_name nvarchar(255);
declare @temp_nonkeyindex_table nvarchar(255);
declare @temp_nonkeyindex_column nvarchar(255);
declare @temp_nonkeyindex_is_unique int;
declare @temp_nonkeyindex_is_unique_constraint int;
declare @temp_nonkeyindex_is_descending nvarchar(10);
declare @temp_nonkey_type_desc nvarchar(255);
declare @alter_column_table table (table_name nvarchar(255)
                                  ,column_name nvarchar(255)
                                  ,datatype nvarchar(255)
                                  ,nullable nvarchar(3));
insert into @alter_column_table (table_name, column_name, datatype, nullable)
select pk_table_name, pk_column_name, pk_datatype, nullable from @upgrade_spec
union
select fk_tc.TABLE_NAME, fk_ccu.COLUMN_NAME, upg.pk_datatype, upg.nullable
from INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS rc,
     INFORMATION_SCHEMA.TABLE_CONSTRAINTS fk_tc,
     INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE fk_ccu,
     INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE rcu,
     @upgrade_spec upg
where rc.CONSTRAINT_CATALOG = @dbname and rc.CONSTRAINT_SCHEMA = @schemaname
and rc.CONSTRAINT_CATALOG = fk_tc.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_tc.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_tc.CONSTRAINT_NAME
and rc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and rc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and rc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME
and fk_tc.CONSTRAINT_CATALOG = fk_ccu.CONSTRAINT_CATALOG and fk_tc.CONSTRAINT_SCHEMA = fk_ccu.CONSTRAINT_SCHEMA and fk_tc.CONSTRAINT_NAME = fk_ccu.CONSTRAINT_NAME
and rc.UNIQUE_CONSTRAINT_CATALOG = rcu.CONSTRAINT_CATALOG and rc.UNIQUE_CONSTRAINT_SCHEMA = rcu.CONSTRAINT_SCHEMA and rc.UNIQUE_CONSTRAINT_NAME = rcu.CONSTRAINT_NAME
and rcu.TABLE_NAME + '|' + rcu.COLUMN_NAME = upg.pk_table_name + '|' + upg.pk_column_name;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '08. Drop foreign keys'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @fk_info;
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
declare cur_alter_column_distinct_tables cursor for select distinct table_name from @alter_column_table;
declare @temp_alter_column_distinct_table_name nvarchar(255);
declare @temp_fk_name nvarchar(255);
declare @temp_fk_table nvarchar(255);
declare @temp_fk_column nvarchar(255);
declare @temp_fk_pk_table nvarchar(255);
declare @temp_fk_pk_column nvarchar(255);
declare @temp_fk_pk_name nvarchar(255);
declare @temp_fk_delete_rule nvarchar(255);
declare @temp_fk_datatype nvarchar(255);
declare cur_fk_tableInfo cursor for select distinct fk_name, fk_table_name, fk_column_name, pk_table_name, pk_column_name, pk_name, fk_delete_rule, datatype from @fk_info;
open cur_fk_tableInfo;
fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_pk_name, @temp_fk_delete_rule, @temp_fk_datatype;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @drop_constraint_cmd nvarchar(255) = 'alter table ' + @temp_fk_table + ' drop constraint ' + @temp_fk_name + ';';
    print '    running drop constraint command: ' + @drop_constraint_cmd;
    exec(@drop_constraint_cmd);
    fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_pk_name, @temp_fk_delete_rule, @temp_fk_datatype;
end
close cur_fk_tableInfo;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '09. Drop other indexes'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @nonkey_indexes
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
declare @dropped_uniqueKeyConstraints table (constraint_name nvarchar(2000)
                                            ,table_name nvarchar(2000)
                                            ,column_name nvarchar(2000));
open cur_nonkey_indexes;
fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
while @@FETCH_STATUS = 0
begin
    if (@temp_nonkeyindex_is_unique_constraint = 1)
    begin
        -- ///////////////////////////////////////////////////////////////////////////
        set @progressSubCurrent = @progressSubCurrent + 1;
        print ''
        print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
        print ''
        -- ///////////////////////////////////////////////////////////////////////////
        declare @drop_unique_constraint_cmd nvarchar(255) = 'alter table ' + @temp_nonkeyindex_table + ' drop constraint ' + @temp_nonkeyindex_name;
        print '    running drop unique constraint command: ' + @drop_unique_constraint_cmd;
        exec(@drop_unique_constraint_cmd);
        insert into @dropped_uniqueKeyConstraints(constraint_name, table_name, column_name)
        values( @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column);
    end;
    fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
end;
close cur_nonkey_indexes;
declare @dropped_nonKeyIndexes table (index_name nvarchar(2000)
                                     ,type_desc nvarchar(2000)
                                     ,table_name nvarchar(2000)
                                     ,column_name nvarchar(2000)
                                     ,is_descending nvarchar(10));
open cur_nonkey_indexes;
fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
while @@FETCH_STATUS = 0
begin
    if (@temp_nonkeyindex_is_unique_constraint = 0)
    begin
        -- ///////////////////////////////////////////////////////////////////////////
        set @progressSubCurrent = @progressSubCurrent + 1;
        print ''
        print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
        print ''
        -- ///////////////////////////////////////////////////////////////////////////
        print '    dropping nonkey index ' + @temp_nonkeyindex_name + ': ' + @temp_nonkeyindex_table + '.' + @temp_nonkeyindex_column;
        declare @drop_nonkey_index_cmd nvarchar(255) = 'drop index ' + @temp_nonkeyindex_name + ' on ' + @temp_nonkeyindex_table + ';';
        print '    running drop index command: ' + @drop_nonkey_index_cmd;
        exec(@drop_nonkey_index_cmd);
        insert into @dropped_nonKeyIndexes(index_name, type_desc, table_name, column_name, is_descending)
        values( @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending);
    end
    fetch next from cur_nonkey_indexes into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_unique, @temp_nonkeyindex_is_unique_constraint, @temp_nonkeyindex_is_descending, @temp_nonkey_type_desc;
end
close cur_nonkey_indexes;
deallocate cur_nonkey_indexes;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '10. Drop primary keys'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @pk_info
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
open cur_pk;
fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @drop_pk_cmd nvarchar(255) = 'alter table ' + @temp_pk_table_name + ' drop constraint ' + @temp_pk_pk_name + ';';
    print '    running drop pk command: ' + @drop_pk_cmd;
    exec(@drop_pk_cmd);
    fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
end
close cur_pk;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '11. Back up the data in the tables containing columns to be altered, determined in step 7 above, by copying their records into temporary tables and then truncating the tables'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(distinct table_name) from @alter_column_table
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
if (@editionID <> '-1592396055' and @editionID <> '-133711905') -- don't do this for 'express editions, as they impose an unchangeable 10 GB database size limit in which case this copying, done for performance, isn't so necessary.
begin
    open cur_alter_column_distinct_tables;
    fetch next from cur_alter_column_distinct_tables into @temp_alter_column_distinct_table_name;
    while @@FETCH_STATUS = 0
    begin
        -- ///////////////////////////////////////////////////////////////////////////
        set @progressSubCurrent = @progressSubCurrent + 1;
        print ''
        print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
        print ''
        -- ///////////////////////////////////////////////////////////////////////////
        declare @backup_cmd nvarchar(255) = 'select * into ' + @temp_alter_column_distinct_table_name + '_dsmupgradetemp from ' + @temp_alter_column_distinct_table_name + ';';
        print '    running backup command: ' + @backup_cmd;
        exec(@backup_cmd);
        declare @truncate_table_cmd nvarchar(255) = 'truncate table ' + @temp_alter_column_distinct_table_name + ';';
        print '    truncate table: ' + @truncate_table_cmd;
        exec(@truncate_table_cmd);
        fetch next from cur_alter_column_distinct_tables into @temp_alter_column_distinct_table_name;
    end
    close cur_alter_column_distinct_tables;
    deallocate cur_alter_column_distinct_tables;
end
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '12. Alter data types of columns from alter_column_spec, the original upgrade spec along with directly referencing external columns'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @alter_column_table
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
declare @temp_alter_column_table nvarchar(255);
declare @temp_alter_column_column nvarchar(255);
declare @temp_alter_column_datatype nvarchar(255);
declare @temp_alter_column_is_nullable nvarchar(3);
declare cur_alter_column cursor for select upg.table_name, upg.column_name, upg.datatype, upg.nullable
from @alter_column_table upg;
open cur_alter_column;
fetch next from cur_alter_column into @temp_alter_column_table, @temp_alter_column_column, @temp_alter_column_datatype, @temp_alter_column_is_nullable;
declare @alter_column_cmd nvarchar(255);
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    set @alter_column_cmd = 'alter table ' + @temp_alter_column_table + ' alter column ' + @temp_alter_column_column + ' ' + @temp_alter_column_datatype;
    if (@temp_alter_column_is_nullable = 'NO' )
    begin
        set @alter_column_cmd = @alter_column_cmd + ' not null';
    end
    set @alter_column_cmd = @alter_column_cmd + ';';
    print '    running ' + @alter_column_cmd;
    exec(@alter_column_cmd);
    fetch next from cur_alter_column into @temp_alter_column_table, @temp_alter_column_column, @temp_alter_column_datatype, @temp_alter_column_is_nullable;
end
close cur_alter_column;
deallocate cur_alter_column;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '13. Restored data that was backed up in step 11 above into the truncated tables containing the columns to be altered as determined in step 7 above'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(a.TABLE_NAME) from INFORMATION_SCHEMA.COLUMNS a where a.TABLE_NAME in (select table_name from @alter_column_table)
set @progressSubTotal = @progressSubTotal + 1;
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
if (@editionID <> '-1592396055' and @editionID <> '-133711905') -- as per step 11 above, don't do this for 'express editions'
begin
    declare cur_columns cursor for
    select a.table_name, a.column_name from INFORMATION_SCHEMA.COLUMNS a
    where a.table_name in (select table_name from @alter_column_table)
    order by table_name, a.ordinal_position;
    declare @insert_table nvarchar(255);
    declare @insert_table_column nvarchar(255);
    declare @insert_table_prev nvarchar(255) = '';
    declare @columns_string nvarchar(4000) = '';
    open cur_columns;
    fetch next from cur_columns into @insert_table, @insert_table_column;
    while @@FETCH_STATUS = 0
    begin
        -- ///////////////////////////////////////////////////////////////////////////
        set @progressSubCurrent = @progressSubCurrent + 1;
        print ''
        print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
        print ''
        -- ///////////////////////////////////////////////////////////////////////////
        if (@insert_table <> @insert_table_prev)
        begin
            if (@insert_table_prev <> '')
            begin

                set @identity_on  = '';
                set @identity_off = '';
                if objectproperty(object_id(@insert_table_prev), 'TableHasIdentity') = 1
                begin
                    set @identity_on  = 'set identity_insert ' + @insert_table_prev + ' on;  ';
                    set @identity_off = ' set identity_insert ' + @insert_table_prev + ' off; '
                                      + ' dbcc checkident  (' + @insert_table_prev + ',reseed); ';
                end
                -- The following string cannot be stored into a variable as its lenght can grow over the t-sql max of 4000 characters
                print 'sqlstring: ' + @identity_on
                    + 'insert into ' + @insert_table_prev + '(' + @columns_string + ')'
                    + ' (select ' + @columns_string + ' from ' + @insert_table_prev + '_dsmupgradetemp); '
                    + @identity_off
                    + ' drop table  ' + @insert_table_prev + '_dsmupgradetemp; ';
                exec( @identity_on
                    + 'insert into ' + @insert_table_prev + '(' + @columns_string + ')'
                    + ' (select ' + @columns_string + ' from ' + @insert_table_prev + '_dsmupgradetemp); '
                    + @identity_off
                    + ' drop table  ' + @insert_table_prev + '_dsmupgradetemp; ');
            end
            set @columns_string = '';
        end
        if (@columns_string <> '' )
        begin
            set @columns_string = @columns_string + ',';
        end
        set @columns_string = @columns_string + '[' + @insert_table_column + ']';
        set @insert_table_prev = @insert_table;
        fetch next from cur_columns into @insert_table, @insert_table_column;
    end;
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    if (@insert_table_prev <> '')
    begin
        set @identity_on  = '';
        set @identity_off = '';
        if objectproperty(object_id(@insert_table_prev), 'TableHasIdentity') = 1
        begin
            set @identity_on  = ' set identity_insert ' + @insert_table_prev + ' on;  ';
            set @identity_off = ' set identity_insert ' + @insert_table_prev + ' off; '
                              + ' dbcc checkident  (' + @insert_table_prev + ',reseed); ';
        end
        -- The following string cannot be stored into a variable as its lenght can grow over the t-sql max of 4000 characters
        print 'sqlstring (final): ' + @identity_on
            + 'insert into ' + @insert_table_prev + '(' + @columns_string + ')'
            + ' (select ' + @columns_string + ' from ' + @insert_table_prev + '_dsmupgradetemp); '
            + @identity_off
            + ' drop table  ' + @insert_table_prev + '_dsmupgradetemp; ';
    end;
    exec( @identity_on
        + 'insert into ' + @insert_table_prev + '(' + @columns_string + ')'
        + ' (select ' + @columns_string + ' from ' + @insert_table_prev + '_dsmupgradetemp); '
        + @identity_off
        + ' drop table  ' + @insert_table_prev + '_dsmupgradetemp; ');
    close cur_columns;
    deallocate cur_columns;
end
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '14. Recreate primary keys'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @pk_info
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
open cur_pk;
fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @create_pk_cmd nvarchar(255) = 'alter table ' + @temp_pk_table_name + ' add constraint ' + @temp_pk_pk_name + ' primary key(' + @temp_pk_column_name + ');';
    print '    running create pk command: ' + @create_pk_cmd;
    exec(@create_pk_cmd);
    fetch next from cur_pk into @temp_pk_pk_name, @temp_pk_table_name, @temp_pk_column_name, @temp_pk_datatype;
end
close cur_pk;
deallocate cur_pk;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '15. Recreate foreign keys'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @fk_info
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
open cur_fk_tableInfo;
fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_name, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_delete_rule, @temp_fk_datatype;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @create_fk_cmd nvarchar(255) = 'alter table ' + @temp_fk_table + ' with check add constraint ' + @temp_fk_name + ' foreign key(' + @temp_fk_column + ') references ' + @temp_fk_pk_table + '(' + @temp_fk_pk_column + ')';
    if (@temp_fk_delete_rule='CASCADE')
    begin
        set @create_fk_cmd = @create_fk_cmd + ' on delete cascade';
    end;
    set @create_fk_cmd = @create_fk_cmd + ';';
    print '    running create fk command: ' + @create_fk_cmd;
    exec(@create_fk_cmd);
    fetch next from cur_fk_tableInfo into @temp_fk_name, @temp_fk_table, @temp_fk_column, @temp_fk_pk_name, @temp_fk_pk_table, @temp_fk_pk_column, @temp_fk_delete_rule, @temp_fk_datatype;
end
close cur_fk_tableInfo;
deallocate cur_fk_tableInfo;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '16. Recreate uniquekeys for other non-primary key indexes'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @dropped_uniqueKeyConstraints
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
declare cur_dropped_uniqueKeyConstraints cursor for
select constraint_name, table_name, column_name from @dropped_uniqueKeyConstraints;
open cur_dropped_uniqueKeyConstraints;
fetch next from cur_dropped_uniqueKeyConstraints into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @create_unique_constraint_cmd nvarchar(255) = 'alter table ' + @temp_nonkeyindex_table + ' add constraint ' + @temp_nonkeyindex_name + ' unique(' + @temp_nonkeyindex_column + ');';
    print '    running create unique constraint command: ' + @create_unique_constraint_cmd;
    exec(@create_unique_constraint_cmd);
    fetch next from cur_dropped_uniqueKeyConstraints into @temp_nonkeyindex_name, @temp_nonkeyindex_table, @temp_nonkeyindex_column;
end
close cur_dropped_uniqueKeyConstraints;
deallocate cur_dropped_uniqueKeyConstraints;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '17. Recreate any leftover nonkey indexes'
-- ///////////////////////////////////////////////////////////////////////////
select @progressSubTotal = count(*) from @dropped_nonKeyIndexes
set @progressSubCurrent = 0;
-- ///////////////////////////////////////////////////////////////////////////
declare cur_dropped_nonkeyIndexes cursor for
select index_name, type_desc, table_name, column_name, is_descending from @dropped_nonKeyIndexes;
open cur_dropped_nonkeyIndexes;
fetch next from cur_dropped_nonkeyIndexes into @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending;
while @@FETCH_STATUS = 0
begin
    -- ///////////////////////////////////////////////////////////////////////////
    set @progressSubCurrent = @progressSubCurrent + 1;
    print ''
    print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' >>> ' + cast(@progressSubCurrent as nvarchar) + '/' + cast(@progressSubTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
    print ''
    -- ///////////////////////////////////////////////////////////////////////////
    declare @create_nonkeyindex_cmd nvarchar(255) = 'create ' + @temp_nonkey_type_desc + ' index ' + @temp_nonkeyindex_name + ' on ' + @temp_nonkeyindex_table + '(' + @temp_nonkeyindex_column + + ' ' + @temp_nonkeyindex_is_descending + ');';
    print '    running create index command: ' + @create_nonkeyindex_cmd;
    exec(@create_nonkeyindex_cmd);
    fetch next from cur_dropped_nonkeyIndexes into @temp_nonkeyindex_name, @temp_nonkey_type_desc, @temp_nonkeyindex_table, @temp_nonkeyindex_column, @temp_nonkeyindex_is_descending;
end
close cur_dropped_nonkeyIndexes;
deallocate cur_dropped_nonkeyIndexes;
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '18. Reset db recovery mode'
-- ///////////////////////////////////////////////////////////////////////////
declare @resetdbrecovery_bulklogged nvarchar(2000) = 'alter database "' + @dbname + '" set recovery ' + @recovery_model;
print ''
print '    Command : ' + @resetdbrecovery_bulklogged;
exec(@resetdbrecovery_bulklogged);
-- ///////////////////////////////////////////////////////////////////////////
set @progressCurrent = @progressCurrent + 1;
print ''
print '*** Current Progress - ' + cast(@progressCurrent as nvarchar) + '/' + cast(@progressTotal as nvarchar) + ' - ' + convert(nvarchar(max), getdate(), 14);
print ''
-- ///////////////////////////////////////////////////////////////////////////
print '19. Schema migration completed successfully'
-- ///////////////////////////////////////////////////////////////////////////
declare @end datetime = getdate();
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '---------------------------------------------------------------------------'
print '@@@ Deep Security Database Migration Script >>> Schema Migration Completed '
print '---------------------------------------------------------------------------'
print '@@@ Completed time : ' + convert(nvarchar(max), @end, 21);
print '---------------------------------------------------------------------------'
print '@@@ Total Schema Migration Time in seconds : ' + cast(datediff(SECOND, @begin, @end) as nvarchar);
print '---------------------------------------------------------------------------'
go

-- ///////////////////////////////////////////////////////////////////////////
-- The main procedure start
-- ///////////////////////////////////////////////////////////////////////////
set nocount on
go

-- ///////////////////////////////////////////////////////////////////////////
-- Define the column specification
declare @altercolumnspec nvarchar(max) = 'systemevents.TargetID:bigint:null|antimalwareevents.AntiMalwareEventID:bigint|antimalwarequarantinedfile.AntiMalwareEventID:bigint|antimalwarespywareitems.AntiMalwareEventID:bigint:null|webreputationevents.WebReputationEventID:bigint|antimalwareeventhistory.AntiMalwareEventHistoryID:bigint|antimalwareeventhistory.AntiMalwareEventID:bigint:null|webreputationeventhistory.WebReputationEventHistoryID:bigint|webreputationeventhistory.WebReputationEventID:bigint:null|integrityevents.IntegrityEventID:bigint|integrityeventhistory.IntegrityEventHistoryID:bigint|integrityeventhistory.IntegrityEventID:bigint:null|loginspectionevents.LogInspectionEventID:bigint|loginspectioneventhistory.LogInspectionEventHistoryID:bigint|loginspectioneventhistory.LogInspectionEventID:bigint:null|packetlogs.PacketLogID:bigint|packetlogdatas.PacketLogDataID:bigint|payloadlogs.PayloadLogID:bigint|payloadlogdatas.PayloadLogDataID:bigint|packetloghistory.PacketLogHistoryID:bigint|packetloghistory.PacketLogID:bigint:null|payloadloghistory.PayloadLogHistoryID:bigint|payloadloghistory.PayloadLogID:bigint:null|entitys.EntityID:bigint|entitys.AgentEntityID:bigint:null|attribute2s.EntityID:bigint|supersededentitys.WasEntityID:bigint';
--declare @altercolumnspec nvarchar(max) = 'systemevents.TargetID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'antimalwareevents.AntiMalwareEventID:bigint|antimalwarequarantinedfile.AntiMalwareEventID:bigint|antimalwarespywareitems.AntiMalwareEventID:bigint:null|antimalwareeventhistory.AntiMalwareEventHistoryID:bigint|antimalwareeventhistory.AntiMalwareEventID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'webreputationevents.WebReputationEventID:bigint|webreputationeventhistory.WebReputationEventHistoryID:bigint|webreputationeventhistory.WebReputationEventID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'integrityevents.IntegrityEventID:bigint|integrityeventhistory.IntegrityEventHistoryID:bigint|integrityeventhistory.IntegrityEventID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'loginspectionevents.LogInspectionEventID:bigint|loginspectioneventhistory.LogInspectionEventHistoryID:bigint|loginspectioneventhistory.LogInspectionEventID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'packetlogs.PacketLogID:bigint|packetlogdatas.PacketLogDataID:bigint|packetloghistory.PacketLogHistoryID:bigint|packetloghistory.PacketLogID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'payloadlogs.PayloadLogID:bigint|payloadlogdatas.PayloadLogDataID:bigint|payloadloghistory.PayloadLogHistoryID:bigint|payloadloghistory.PayloadLogID:bigint:null';
--declare @altercolumnspec nvarchar(max) = 'entitys.EntityID:bigint|entitys.AgentEntityID:bigint:null|attribute2s.EntityID:bigint|supersededentitys.WasEntityID:bigint';
-- ///////////////////////////////////////////////////////////////////////////
-- Define the deprecated table check list
declare @deprecated_table_list nvarchar(max) = 'attributes|attributes_dsmupgradetemp';
-- ///////////////////////////////////////////////////////////////////////////
declare @upgrade_spec as tbl_upgrade_spec;
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '---------------------------------------------------------------------------'
print '@@@ Deep Security Database Migration Script >>> Build Upgrade Specification'
print '---------------------------------------------------------------------------'
-- ///////////////////////////////////////////////////////////////////////////
declare @altercolumnentry nvarchar(512);
declare @altercolumnspecidx int = len(@altercolumnspec);
while (@altercolumnspecidx > 0)
begin
    set @altercolumnspecidx = charindex('|', @altercolumnspec);
    if (@altercolumnspecidx = 0)
    begin
        set @altercolumnentry = @altercolumnspec;
        set @altercolumnspec = '';
    end
    else
    begin
        set @altercolumnentry = substring(@altercolumnspec,1,@altercolumnspecidx-1);
        set @altercolumnspec = substring(@altercolumnspec,@altercolumnspecidx+1,len(@altercolumnspec)-@altercolumnspecidx);
    end
    print ''
    print '    current  : ' + @altercolumnentry;
    print '    remainder: ' + @altercolumnspec;
    declare @tableEndPos int = charindex('.', @altercolumnentry);
    declare @columnEndPos int = charindex(':', @altercolumnentry);
    print '    tableEndPos: ' + cast(@tableEndPos as nvarchar(255));
    print '    columnEndPos: ' + cast(@columnEndPos as nvarchar(255));
    declare @altercolumnentry_tableName nvarchar(127) = substring(@altercolumnentry,1,@tableEndPos-1);
    declare @altercolumnentry_columnName nvarchar(127) = substring(@altercolumnentry,@tableEndPos+1,@columnEndPos-@tableEndPos-1);
    declare @altercolumnentry_datatype nvarchar(127) = substring(@altercolumnentry,@columnEndPos+1,len(@altercolumnentry)-@columnEndPos);
    declare @altercolumnentry_nullable nvarchar(3) = 'NO';
    if (substring(@altercolumnentry_datatype,len(@altercolumnentry_datatype)-4, 5) = ':null')
    begin
        set @altercolumnentry_nullable = 'YES';
        set @altercolumnentry_datatype = substring(@altercolumnentry_datatype,1, len(@altercolumnentry_datatype)-5);
    end
    -- ///////////////////////////////////////////////////////////////////////////
    if exists (select TABLE_NAME from INFORMATION_SCHEMA.COLUMNS where DATA_TYPE <> @altercolumnentry_datatype and TABLE_NAME = @altercolumnentry_tableName and COLUMN_NAME = @altercolumnentry_columnName)
    begin
        insert into @upgrade_spec(pk_table_name, pk_column_name, pk_datatype, nullable)
        values(@altercolumnentry_tableName,@altercolumnentry_columnName,@altercolumnentry_datatype, @altercolumnentry_nullable);
    end
    else
    begin
        print '    ' + 'The datatype of column ' + @altercolumnentry_columnName + ' in table ' + @altercolumnentry_tableName + ' has already been ' + @altercolumnentry_datatype
        print '    ' + 'Skip this item'
    end
    -- ///////////////////////////////////////////////////////////////////////////
end
-- ///////////////////////////////////////////////////////////////////////////
-- Check deprecated table "attributes" and remove it if exists
-- ///////////////////////////////////////////////////////////////////////////
print ''
print '---------------------------------------------------------------------------'
print '@@@ Deep Security Database Migration Script >>> Check Deprecated Tables    '
print '---------------------------------------------------------------------------'
-- ///////////////////////////////////////////////////////////////////////////
declare @entry nvarchar(512);
declare @index int = len(@deprecated_table_list);
while (@index > 0)
begin
    set @index = charindex('|', @deprecated_table_list);
    if (@index = 0)
    begin
        set @entry = @deprecated_table_list;
        set @deprecated_table_list = '';
    end
    else
    begin
        set @entry = substring(@deprecated_table_list,1,@index-1);
        set @deprecated_table_list = substring(@deprecated_table_list,@index+1,len(@deprecated_table_list)-@index);
    end
    print ''
    print '    ' + 'current  : ' + @entry;
    print '    ' + 'remainder: ' + @deprecated_table_list;
    -- ///////////////////////////////////////////////////////////////////////////
    if exists (select 1 from sys.objects where object_id = OBJECT_ID(@entry) and type = 'U')
    begin
        print ''
        print '    ' + 'The deprecated table ' + @entry + ' exists! Drop this deprecated table to prevent schema update problem.'
        declare @cmd_drop_table_deprecated nvarchar(255) = 'drop table ' + @entry;
        print '    ' + @cmd_drop_table_deprecated;
        exec(@cmd_drop_table_deprecated);
    end
    else
    begin
        print ''
        print '    ' + 'The deprecated table ' + @entry + ' does not exist. Continue to schema update.'
    end
    -- ///////////////////////////////////////////////////////////////////////////
end
-- ///////////////////////////////////////////////////////////////////////////
if exists (select pk_table_name from @upgrade_spec)
begin
    exec GenerateRecoveryCommand @upgrade_spec
    exec UpdateTableSchema @upgrade_spec
end
else
begin
    print ''
    print '    ' + 'There are no items in upgrade specification table. No need to do any update on current database'
end
go

-- ///////////////////////////////////////////////////////////////////////////
-- Drop store procedure before we leave
drop procedure GenerateRecoveryCommand;
drop procedure UpdateTableSchema;
drop type tbl_upgrade_spec;
go

-- ///////////////////////////////////////////////////////////////////////////
set nocount off
go














print '  ';
print '  ';
print '  ';
print '  ';
print '**************** Finished Executing Migration Script ****************';
print '  ';
print '  ';
print '  ';
print '  ';
GO






USE [master];
GO

declare @execSql varchar(2000);
declare @databaseName varchar(100);
SET @databaseName = '[$(TENANT_DB_NAME_NEW)]';
SET @execSql = '';
SELECT  @execSql = @execSql + 'KILL ' + cast(spid as varchar(100)) + '; ' FROM    master.dbo.sysprocesses WHERE (db_name(dbid) = @databaseName AND DBID <> 0 AND spid <> @@spid);
SET @execSql = @execSql + '; ';

SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME_NEW)] SET SINGLE_USER WITH ROLLBACK IMMEDIATE; '; 
SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME_NEW)] MODIFY NAME = [$(TENANT_DB_NAME)]; '; 
SET @execSql = @execSql + ' ALTER DATABASE [$(TENANT_DB_NAME)] SET MULTI_USER WITH ROLLBACK IMMEDIATE; ';

print '  ';
print '  ';
print '---------------------------------------- Reverted DB Name [$(TENANT_DB_NAME)] ----------------------------------------';
print '  ';
print '  ';

EXEC(@execSql);
GO


EXEC sp_configure 'remote query timeout', '$(REMOTE_QUERY_TIMEOUT)';
GO
RECONFIGURE;
GO

print '---------------------------------------- Sending Quit to SQLCMD ----------------------------------------';
GO

:quit


